from rest_framework import status


class GizwitsException(Exception):
    status = status.HTTP_400_BAD_REQUEST
    code = 'xxxxxx'
    old_code = None  # 兼容 gwapi 的错误码
    old_msg = None  # 兼容 gwapi 的错误信息
    message = 'Not specified'
    description = '机智云的错误大类'
    detail = None

    def __init__(self, detail=None, code=None, message=None) -> None:
        """

        :param code: 特殊情况下，需要修改 code
        :param message: 特殊情况下，需要修改 message
        :param detail: 错误的详细信息
        """
        self.detail = detail

        if code is not None:
            self.code = code

        if message is not None:
            self.message = message


class SystemException(GizwitsException):
    status = status.HTTP_500_INTERNAL_SERVER_ERROR
    code = '00xxxx'
    description = '系统性错误，如依赖服务报错'


class DataException(GizwitsException):
    status = status.HTTP_400_BAD_REQUEST
    code = '01xxxx'
    description = '数据性错误'


class BusinessException(GizwitsException):
    status = status.HTTP_400_BAD_REQUEST
    code = '02xxxx'
    description = '业务性错误'



from rest_framework import status

from .base import DataException


class DeviceNotFound(DataException):
    status = status.HTTP_404_NOT_FOUND

    old_code = 9014
    old_msg = 'device not found!'

    message = 'Device not found'
    code = '010101'


class DeviceDisabled(DataException):
    old_code = 9023
    old_msg = 'device is disabled!'

    message = 'Device is disabled'
    code = '010102'


class ProductNotFound(DataException):
    status = status.HTTP_404_NOT_FOUND
    message = 'Product not found'
    code = '010201'


class ProductProvisionDoesNotExists(DataException):
    message = 'Product provision does not exists'
    code = '010202'


class ProductMultipleRecord(DataException):
    message = 'Multiple product found'
    code = '010203'



from rest_framework.exceptions import _get_error_details

from giz_exceptions.paas.exceptions import BusinessException


class SerializerValidationError(BusinessException):
    old_code = 9015
    old_msg = 'form_invalid'

    message = 'Form invalid'
    code = '020001'

    def __init__(self, detail=None):

        if isinstance(detail, dict):
            self.detail = _get_error_details(detail, 'invalid')

        if isinstance(detail, str):
            self.detail = detail


from .base import *
from .business import *
from .data import *
from .system import *


def all_subclasses(cls):
    return set(cls.__subclasses__()).union(
        [
            leaf_cls
            for sub_cls in cls.__subclasses__()
            for leaf_cls in all_subclasses(sub_cls)
        ]
    )


ERRORS = {
    cls.code: cls
    for cls in all_subclasses(GizwitsException)
}



from giz_exceptions.paas.exceptions import BusinessException


class DeviceRegisterLimit(BusinessException):
    old_code = 9055
    code = '020103'
    message = 'Devices count has reached the limit of product,' \
              ' please contact sellers!'


class SlaveDeviceBidingLimit(BusinessException):
    message = 'Slave device can no more bind to current central control device'
    code = '020104'


class DeviceUpdateFailed(BusinessException):
    message = 'Device update failed'
    code = '020105'


class DeviceResetFailed(BusinessException):
    message = 'Device reset failed'
    code = '020106'


class DeviceCreateFailed(BusinessException):
    message = 'Device create failed'
    code = '020107'


class DeviceAssociateFailed(BusinessException):
    message = 'Device associate failed'
    code = '020108'



from giz_exceptions.paas.exceptions import BusinessException


class ProductKeyInvalid(BusinessException):
    old_code = 9002
    old_msg = 'product_key invalid!'

    message = 'Product key invalid'
    code = '020201'


class ProductTypeError(BusinessException):
    message = 'Product type error'
    code = '020202'


class ProductConfigError(BusinessException):
    message = 'Product configuration error'
    code = '020203'


class ProductPreAssignmentFailed(BusinessException):
    message = 'Product pre-assignment failed'
    code = '020204'



from rest_framework.response import Response
from rest_framework.views import exception_handler

from .paas.exceptions.base import GizwitsException


def log(context, **kwargs):
    request = context.get('request')
    if request and hasattr(request, 'biz_log'):
        request.biz_log.update(kwargs)


def custom_handler(exc, context):
    """

    异常的处理优先级：
    1. 优先处理自定义异常
    2. 非自定义异常，交由

    :param exc: 异常对象
    :param context: 字典，包含：view、args、kwargs、request
    :return:
    """

    if not isinstance(exc, GizwitsException):

        response = exception_handler(exc, context)

        if response:
            return response

        return

    data = {
        'error_code': exc.old_code or exc.code,
        'error_message': exc.old_msg or exc.message,
        'detail_message': exc.detail,
    }

    log(context, http_status=exc.status, **data)

    return Response(data, status=exc.status)



from giz_exceptions.paas.exceptions import BusinessException


class GDCSExceptions(BusinessException):
    message = 'GDCS error'
    code = '020400'


class DeviceNotYetReport(GDCSExceptions):
    message = 'current device not yet reported'
    code = '020401'


class GDCSHadExpired(GDCSExceptions):
    message = 'the GDCS of current device is expired'
    code = '020402'


class GDCSOutOfStock(GDCSExceptions):
    message = 'the GDCS is out of stock'
    code = '020403'
